#!/bin/env python
from pkg_resources import parse_version
import kaitaistruct
from kaitaistruct import KaitaiStruct, KaitaiStream, BytesIO

if parse_version(kaitaistruct.__version__) < parse_version('0.9'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.9 or later is required, but you have %s" % (kaitaistruct.__version__))

class TwDatafileV4(KaitaiStruct):
    """
    .. seealso::
       Source - https://github.com/heinrich5991/libtw2/blob/b510f20bc58ceb33f38ddc555b63989ccf5a90d7/doc/datafile.md
    """
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.header = TwDatafileV4.Header(self._io, self, self._root)
        self.item_types = [None] * (self.header.num_item_types)
        for i in range(self.header.num_item_types):
            self.item_types[i] = TwDatafileV4.ItemType(self._io, self, self._root)

        self.item_offsets = [None] * (self.header.num_items)
        for i in range(self.header.num_items):
            self.item_offsets[i] = self._io.read_s4le()

        self.data_offsets = [None] * (self.header.num_data)
        for i in range(self.header.num_data):
            self.data_offsets[i] = self._io.read_s4le()

        self.data_sizes = [None] * (self.header.num_data)
        for i in range(self.header.num_data):
            self.data_sizes[i] = self._io.read_s4le()

        self.items = [None] * (self.header.num_items)
        for i in range(self.header.num_items):
            self.items[i] = TwDatafileV4.Item(self._io, self, self._root)

        self.data = [None] * (self.header.num_data)
        for i in range(self.header.num_data):
            self.data[i] = self._io.read_bytes(((self.header.data_size if i == (self.header.num_data - 1) else self.data_offsets[(i + 1)]) - self.data_offsets[i]))


    class Header(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.magic = self._io.read_bytes(4)
            if not self.magic == b"\x44\x41\x54\x41":
                raise kaitaistruct.ValidationNotEqualError(b"\x44\x41\x54\x41", self.magic, self._io, u"/types/header/seq/0")
            self.version = self._io.read_bytes(4)
            if not self.version == b"\x04\x00\x00\x00":
                raise kaitaistruct.ValidationNotEqualError(b"\x04\x00\x00\x00", self.version, self._io, u"/types/header/seq/1")
            self.size = self._io.read_s4le()
            self.swaplen = self._io.read_s4le()
            self.num_item_types = self._io.read_s4le()
            self.num_items = self._io.read_s4le()
            self.num_data = self._io.read_s4le()
            self.item_size = self._io.read_s4le()
            self.data_size = self._io.read_s4le()


    class ItemType(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.type_id = self._io.read_s4le()
            self.start = self._io.read_s4le()
            self.num = self._io.read_s4le()


    class Item(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self._read()

        def _read(self):
            self.id = self._io.read_u2le()
            self.type_id = self._io.read_u2le()
            self.size = self._io.read_s4le()
            self.item_data = [None] * (self.size // 4)
            for i in range(self.size // 4):
                self.item_data[i] = self._io.read_s4le()

# Parser above compiled by Kaitai, using https://github.com/heinrich5991/libtw2/blob/be7ff357b1567f923f9d168c59b94c872106d4d1/doc/datafile_v4.ksy
# Script below written by Patiga

import sys
from pathlib import Path
if len(sys.argv) != 2:
    print("Please pass the path to a map as a command line argument")
    sys.exit(1)
map = TwDatafileV4.from_file(sys.argv[1])

image_items = [item for item in map.items if item.type_id == 2]
image_indices = [item.item_data[5] for item in image_items]
image_sizes = []
for i in image_indices:
    if i == -1:
        image_sizes.append(0)
    else:
        image_sizes.append(len(map.data[i]))
for i, size in enumerate(image_sizes):
    print(f"image {i+1:2}: {size}")
